/*
Under GNU License 2

Read copying in the root directory of the distribution
If you do not have copying, contact me because you should 
have received it with this software package, the author, 
or you can view the license at <http://www.gnu.org>
*/
//import java.lang.*;
import java.net.InetAddress;
import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.UnknownHostException;
import java.net.SocketException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.StringTokenizer;

/**
Mark's TFtp Client<BR>
Trival File Transfer Protocol (RFC 1350)<BR>
<BR>
Mark Benvenuto <ivymccough@worldnet.att.net><BR>
Java JDK 1.1.5<BR>
<BR>
A TFTP client conforming to TFTP prtocol version 2 (RFC 1360)<BR>
Under GNU License Version 2.<BR>
<BR>
Version History<BR>
<BR>
	Check NEWS & ChangeLog files<BR>

@author Mark Benvenuto
@version 0.4
@see MtftpConnection

*/
public class Mtftp
{
	int debug = 0;
	static final short TFTP_ReadRequest = 1;
	static final short TFTP_WriteRequest = 2;
	/**
		Process the command line arguements and calls MTftpConnection

		@param args The arguements from the command line
		@see MtftpConnection
	*/
	Mtftp(String[] args)
	{
		int i, a=0, requesttype=0;
		String addr = null;
		String rFilen, lFilen;
		InetAddress ia;
		MtftpConnection conn;

		//Display Welcome banner and version
		System.out.println("Welcome to Mark's TFTP client\nVersion 0.4");

		//Process arguements
		for(i = 0; i < args.length; i++)
		{
			String arg = args[i];
			if(arg.startsWith("-version"))
			{
				System.out.println("Mark's TFTP client Version 0.4\nMark Benvenuto");
				return;
			}
			if(arg.startsWith("-help"))
			{
				usage();
				return;
			}
			if(arg.startsWith("-debug") || arg.startsWith("-verbose") )
			{
				debug = 1;
				a = 1;
			}
			//if(arg.startsWith("-addr"))
			//	addr = args[++i];
		}
		try{

			if( ((debug == 1) && ( args.length == 2)) || ((debug != 1) && (args.length == 1)) )
			{
				//We have one token (besides -debug)
				//and it should be the address of the remote computer
				//with a filename
				//Format is 'host:file'

				//This is a read request
				requesttype = TFTP_ReadRequest;

				StringTokenizer st = new StringTokenizer(args[a], " :", false);

				if(st.countTokens() != 2)
				{
					System.out.println("Invalid Format for file name, must be \'remotehost:file\'");
					return;
				}

				//Get address of host
				ia = InetAddress.getByName(st.nextToken());

				//Get remote and local file name
				lFilen = rFilen = st.nextToken();
			}
			else if( ((debug == 1) && ( args.length == 3)) || ((debug != 1) && (args.length == 2)) )
			{
				//We have multiple token (besides -debug)
				//and it should be the address and filename pairs
				//format is 'remotehost:file' 'file'
				//or format is 'file' 'remotehost:file'

				String tstr = args[a] + " " + args[a + 1];
				StringTokenizer st = new StringTokenizer(tstr, " ", false);

				if( st.countTokens() != 2)
				{
					System.out.println("Invalid Format for command parameters,\nmust be \'remotehost:file file\' or \'file remotehost:file\'");
					return;
				}
				
				//Get the tokens
				String t1 = st.nextToken(), t2 = st.nextToken();

				st = new StringTokenizer(t1, " :", false);
				if( st.countTokens() == 2) 
				{
					//We must have the remotehost:file
					
					//This is a read request
					requesttype = TFTP_ReadRequest;

					//Get address of host
					ia = InetAddress.getByName(st.nextToken());

					//Get remote and local file name
					rFilen = st.nextToken();

					lFilen = t2;
				}
				else
				{
					st = new StringTokenizer(t2, " :", false);
					if( st.countTokens() == 2) 
					{
						//We must have the remotehost:file

						//This is a write request
						requesttype = TFTP_WriteRequest;

						//Get address of host
						ia = InetAddress.getByName(st.nextToken());

						//Get remote and local file name
						rFilen = st.nextToken();

						lFilen = t1;
					}
					else
					{
						System.out.println("Invalid Format for individual parameters,\none must be \'remotehost:file\'");
						return;
					}
				}

			}
			else if(args.length == 0)
			{
				System.out.println("Too few parameters");
				usage();
				return;
			}
			else
			{
				System.out.println("Too many parameters");
				usage();
				return;
			}

			if(debug > 0)
			{
				System.out.println("Opening address: " + ia);
				System.out.println("Processing File: " + rFilen);
			}

			//Create new Tftp Connection
			conn = new MtftpConnection(ia, rFilen, lFilen, requesttype, debug);

			//Set the socket timeout to 3 seconds
			conn.setSocketTimeout(3000);

			//Process the Tftp Connection
			conn.processConnection();
		
		}
		catch(UnknownHostException uhe)
		{
			System.err.println("Could not Resolve Local Host name: " + uhe);
		}
		
	}
	/**
		Displays command line options
	*/
	void usage()
	{
		System.out.println("usage: remotehost:file [file] or file remotehost:file\n"
				  +" -debug        Same as -verbose\n"
				  +" -help         This is the help\n"
				  +" -version      Displays version information\n"
				  +" -verbose      Displays additional information during transfers\n\n"
				  +"For additional help contact Mark Benvenuto <ivymccough@worldnet.att.net>"
				);
	}
	/**
		Creates an instance of MTftp and passes the 
		current command line parameters to it

		@see Mtftp
	*/
	public static void main(String[] args)
	{
		Mtftp client;
		client = new Mtftp(args);
	}
}
